
/*
======================COPYRIGHT/LICENSE START==========================

contour_style.c: Part of the CcpNmr Analysis program

Copyright (C) 2003-2010 Wayne Boucher and Tim Stevens (University of Cambridge)

=======================================================================

The CCPN license can be found in ../../../license/CCPN.license.

======================COPYRIGHT/LICENSE END============================

for further information, please contact :

- CCPN website (http://www.ccpn.ac.uk/)

- email: ccpn@bioc.cam.ac.uk

- contact the authors: wb104@bioc.cam.ac.uk, tjs23@cam.ac.uk
=======================================================================

If you are using this software for academic purposes, we suggest
quoting the following references:

===========================REFERENCE START=============================
R. Fogh, J. Ionides, E. Ulrich, W. Boucher, W. Vranken, J.P. Linge, M.
Habeck, W. Rieping, T.N. Bhat, J. Westbrook, K. Henrick, G. Gilliland,
H. Berman, J. Thornton, M. Nilges, J. Markley and E. Laue (2002). The
CCPN project: An interim report on a data model for the NMR community
(Progress report). Nature Struct. Biol. 9, 416-418.

Wim F. Vranken, Wayne Boucher, Tim J. Stevens, Rasmus
H. Fogh, Anne Pajon, Miguel Llinas, Eldon L. Ulrich, John L. Markley, John
Ionides and Ernest D. Laue (2005). The CCPN Data Model for NMR Spectroscopy:
Development of a Software Pipeline. Proteins 59, 687 - 696.

===========================REFERENCE END===============================
*/
#include "contour_style.h"

#define  NCOLORS  3

Contour_style new_contour_style(int npos_colors, float **pos_colors,
				int nneg_colors, float **neg_colors,
				int pos_line_style, int neg_line_style)
{
    Contour_style contour_style;

    MALLOC_NEW(contour_style, struct Contour_style, 1);

    contour_style->npos_colors = npos_colors;
    contour_style->pos_colors = pos_colors;
    contour_style->nneg_colors = nneg_colors;
    contour_style->neg_colors = neg_colors;
    contour_style->pos_line_style = pos_line_style;
    contour_style->neg_line_style = neg_line_style;

    return contour_style;
}
 
void delete_contour_style(Contour_style contour_style)
{
    int i;

    if (contour_style)
    {
	for (i = 0; i < contour_style->npos_colors; i++)
	    FREE(contour_style->pos_colors[i], float);
	FREE(contour_style->pos_colors, float *);

	for (i = 0; i < contour_style->nneg_colors; i++)
	    FREE(contour_style->neg_colors[i], float);
	FREE(contour_style->neg_colors, float *);
    }

    FREE(contour_style, struct Contour_style);
}
